# Generated by Django 3.2.11 on 2022-01-26 18:37

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('sap_success_factors', '0008_alter_sapsuccessfactorsenterprisecustomerconfiguration_enterprise_customer'),
    ]

    operations = [
        migrations.AlterField(
            model_name='sapsuccessfactorsenterprisecustomerconfiguration',
            name='catalogs_to_transmit',
            field=models.TextField(blank=True, default='', help_text='A comma-separated list of catalog UUIDs to transmit. If blank, all customer catalogs will be transmitted. If there are overlapping courses in the customer catalogs, the overlapping course metadata will be selected from the newest catalog.'),
        ),
        migrations.AlterField(
            model_name='sapsuccessfactorsenterprisecustomerconfiguration',
            name='channel_worker_username',
            field=models.CharField(blank=True, default='', help_text='Enterprise channel worker username to get JWT tokens for authenticating LMS APIs.', max_length=255),
        ),
        migrations.AlterField(
            model_name='sapsuccessfactorsenterprisecustomerconfiguration',
            name='idp_id',
            field=models.CharField(blank=True, default='', help_text='If provided, will be used as IDP slug to locate remote id for learners', max_length=255),
        ),
        migrations.AlterField(
            model_name='sapsuccessfactorsenterprisecustomerconfiguration',
            name='key',
            field=models.CharField(blank=True, default='', help_text='OAuth client identifier.', max_length=255, verbose_name='Client ID'),
        ),
        migrations.AlterField(
            model_name='sapsuccessfactorsenterprisecustomerconfiguration',
            name='sapsf_base_url',
            field=models.CharField(blank=True, default='', help_text='Base URL of success factors API.', max_length=255, verbose_name='SAP Base URL'),
        ),
        migrations.AlterField(
            model_name='sapsuccessfactorsenterprisecustomerconfiguration',
            name='sapsf_company_id',
            field=models.CharField(blank=True, default='', help_text='Success factors company identifier.', max_length=255, verbose_name='SAP Company ID'),
        ),
        migrations.AlterField(
            model_name='sapsuccessfactorsenterprisecustomerconfiguration',
            name='sapsf_user_id',
            field=models.CharField(blank=True, default='', help_text='Success factors user identifier.', max_length=255, verbose_name='SAP User ID'),
        ),
        migrations.AlterField(
            model_name='sapsuccessfactorsenterprisecustomerconfiguration',
            name='secret',
            field=models.CharField(blank=True, default='', help_text='OAuth client secret.', max_length=255, verbose_name='Client Secret'),
        ),
    ]
