# Generated by Django 3.2.11 on 2022-02-14 16:27

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('degreed2', '0005_auto_20220131_1539'),
    ]

    operations = [
        migrations.AlterField(
            model_name='degreed2enterprisecustomerconfiguration',
            name='transmission_chunk_size',
            field=models.IntegerField(default=1, help_text='The maximum number of data items to transmit to the integrated channel with each request.'),
        ),
        migrations.AlterField(
            model_name='historicaldegreed2enterprisecustomerconfiguration',
            name='transmission_chunk_size',
            field=models.IntegerField(default=1, help_text='The maximum number of data items to transmit to the integrated channel with each request.'),
        ),
    ]
