# Generated by Django 2.2.16 on 2020-10-20 06:42

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0113_auto_20200914_2054'),
    ]

    operations = [
        migrations.AddField(
            model_name='enterprisecustomer',
            name='default_language',
            field=models.CharField(blank=True, choices=[('en', 'English'), ('es-419', 'Español (Latinoamérica)')], default=None, help_text='Specifies the default language for all the learners of this enterprise customer.', max_length=25, null=True),
        ),
        migrations.AddField(
            model_name='historicalenterprisecustomer',
            name='default_language',
            field=models.CharField(blank=True, choices=[('en', 'English'), ('es-419', 'Español (Latinoamérica)')], default=None, help_text='Specifies the default language for all the learners of this enterprise customer.', max_length=25, null=True),
        ),
    ]
